/*
	FreeRTOS.org V5.1.0 - Copyright (C) 2003-2008 Richard Barry.

	This file is part of the FreeRTOS.org distribution.

	FreeRTOS.org is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	FreeRTOS.org is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with FreeRTOS.org; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	A special exception to the GPL can be applied should you wish to distribute
	a combined work that includes FreeRTOS.org, without being obliged to provide
	the source code for any proprietary components.  See the licensing section
	of http://www.FreeRTOS.org for full details of how and when the exception
	can be applied.

    ***************************************************************************
    ***************************************************************************
    *                                                                         *
    * SAVE TIME AND MONEY!  We can port FreeRTOS.org to your own hardware,    *
    * and even write all or part of your application on your behalf.          *
    * See http://www.OpenRTOS.com for details of the services we provide to   *
    * expedite your project.                                                  *
    *                                                                         *
    ***************************************************************************
    ***************************************************************************

	Please ensure to read the configuration and relevant port sections of the
	online documentation.

	http://www.FreeRTOS.org - Documentation, latest information, license and
	contact details.

	http://www.SafeRTOS.com - A version that is certified for use in safety
	critical systems.

	http://www.OpenRTOS.com - Commercial support, development, porting,
	licensing and training services.


 Changes for this instance of portmacro.h only,  by Paul Murphy 2008

 + Replaced

        #define portENTER_CRITICAL()        POSTINC1 = INTCON;                     \
                                            INTCONbits.GIEH = 0;

   with

        #define portENTER_CRITICAL()        vPortEnterCritical()


 + Replaced

        #define portEXIT_CRITICAL()         _asm                                    \
                                                 MOVF	POSTDEC1, 1, 0              \
                                            _endasm	                                \
                                            if( INDF1 & portGLOBAL_INT_ENABLE_BIT )	\
                                            {                                       \
										        portENABLE_INTERRUPTS();            \
									        }

   with

       #define portEXIT_CRITICAL()          vPortExitCritical()

*/


#ifndef PORTMACRO_H
#define PORTMACRO_H

#include "FreeRTOSConfig.h"

#include "projdefs.h"

/************************************************************************************************************/

/* Type definitions. */

#define portCHAR        		char
#define portFLOAT       		float
#define portDOUBLE      		double
#define portLONG        		long
#define portSHORT       		int
#define portSTACK_TYPE  		unsigned char
#define portBASE_TYPE   		char

typedef unsigned portSHORT		portTickType;

#define portMAX_DELAY 			( portTickType ) 0xffff

/************************************************************************************************************/

/* Hardware specifics. */

#define portBYTE_ALIGNMENT      1
#define portSTACK_GROWTH        1
#define portTICK_RATE_MS        ( ( portTickType ) 1000 / configTICK_RATE_HZ )	// Not needed for VEX robotics

/************************************************************************************************************/

/* Task utilities. */

extern void vPortYield( void );

#define portYIELD()             vPortYield()

/************************************************************************************************************/

/* Task function macros as described on the FreeRTOS.org WEB site. */

#define portTASK_FUNCTION_PROTO( vFunction, pvParameters ) void vFunction( void *pvParameters )

#define portTASK_FUNCTION( vFunction, pvParameters ) void vFunction( void *pvParameters )

/************************************************************************************************************/

/* Compiler specifics. */

#define inline

/************************************************************************************************************/

/* Critical section management - Disabling and Re-enabling of interrupts and tracking interrupt nesting 	*/

#define portDISABLE_INTERRUPTS()				INTCONbits.GIEL = 0;

#define portENABLE_INTERRUPTS()					INTCONbits.GIEL = 1;

#define portENTER_CRITICAL()					vPortEnterCritical()

#define portEXIT_CRITICAL()						vPortExitCritical()

/************************************************************************************************************/

#endif /* PORTMACRO_H */

